@@ -11,11 +11,15 @@
 */
 
 #define _BSD_SOURCE  /* Make sure string.h contains strdup() */
+#define _XOPEN_SOURCE 500  /* Make sure strdup() is in string.h */
+
 #include <string.h>
 
 #include "pam.h"
 #include "pammap.h"
 #include "shhopt.h"
+#include "mallocvar.h"
+#include "nstring.h"
 #include "tga.h"
 
 /* Max number of colors allowed for colormapped output. */
@@ -107,31 +111,31 @@ writeTgaHeader(struct ImageHeader const tgaHeader) {
 
     unsigned char flags;
 
-    putchar( tgaHeader.IDLength );
-    putchar( tgaHeader.CoMapType );
-    putchar( tgaHeader.ImgType );
-    putchar( tgaHeader.Index_lo );
-    putchar( tgaHeader.Index_hi );
-    putchar( tgaHeader.Length_lo );
-    putchar( tgaHeader.Length_hi );
-    putchar( tgaHeader.CoSize );
-    putchar( tgaHeader.X_org_lo );
-    putchar( tgaHeader.X_org_hi );
-    putchar( tgaHeader.Y_org_lo );
-    putchar( tgaHeader.Y_org_hi );
-    putchar( tgaHeader.Width_lo );
-    putchar( tgaHeader.Width_hi );
-    putchar( tgaHeader.Height_lo );
-    putchar( tgaHeader.Height_hi );
-    putchar( tgaHeader.PixelSize );
-    flags = ( tgaHeader.AttBits & 0xf ) | 
-        ( ( tgaHeader.Rsrvd & 0x1 ) << 4 ) |
-        ( ( tgaHeader.OrgBit & 0x1 ) << 5 ) | 
-        ( ( tgaHeader.OrgBit & 0x3 ) << 6 );
-    putchar( flags );
+    putchar(tgaHeader.IdLength);
+    putchar(tgaHeader.CoMapType);
+    putchar(tgaHeader.ImgType);
+    putchar(tgaHeader.Index_lo);
+    putchar(tgaHeader.Index_hi);
+    putchar(tgaHeader.Length_lo);
+    putchar(tgaHeader.Length_hi);
+    putchar(tgaHeader.CoSize);
+    putchar(tgaHeader.X_org_lo);
+    putchar(tgaHeader.X_org_hi);
+    putchar(tgaHeader.Y_org_lo);
+    putchar(tgaHeader.Y_org_hi);
+    putchar(tgaHeader.Width_lo);
+    putchar(tgaHeader.Width_hi);
+    putchar(tgaHeader.Height_lo);
+    putchar(tgaHeader.Height_hi);
+    putchar(tgaHeader.PixelSize);
+    flags = (tgaHeader.AttBits & 0xf) | 
+        ((tgaHeader.Rsrvd & 0x1) << 4) |
+        ((tgaHeader.OrgBit & 0x1) << 5) | 
+        ((tgaHeader.OrgBit & 0x3) << 6);
+    putchar(flags);
 
-    if ( tgaHeader.IDLength > 0 )
-        fwrite( tgaHeader.id, 1, (int) tgaHeader.IDLength, stdout );
+    if (tgaHeader.IdLength > 0)
+        fwrite(tgaHeader.Id, 1, (int) tgaHeader.IdLength, stdout);
 }
     
 
@@ -140,8 +144,14 @@ static void
 putPixel(struct pam *          const pamP,
          tuple                 const tuple, 
          enum TGAbaseImageType const imgType, 
+         bool                  const withAlpha,
          tuplehash             const cht) {
-
+/*----------------------------------------------------------------------------
+   Write a single pixel of the TGA raster to Standard Output.  The
+   pixel is to have color 'tuple'.  The raster has format 'imgType'.
+   The color palette from which the specified color is to be drawn, if
+   'imgType' indicates use of a color palette, is 'cht'.
+-----------------------------------------------------------------------------*/
     if (imgType == TGA_MAP_TYPE) {
         int retval;
         int found;
@@ -156,7 +166,7 @@ putPixel(struct pam *          const pamP,
             /* Make RGB pixel out of a single input plane */
             unsigned int plane;
             
-            for (plane = 0; plane < pamP->depth; ++plane) 
+            for (plane = 0; plane < 3; ++plane) 
                 putchar(pnm_scalesample(tuple[0], 
                                         pamP->maxval, TGA_MAXVAL));
         } else if (imgType == TGA_MONO_TYPE)
@@ -169,7 +179,7 @@ putPixel(struct pam *          const pamP,
                                     pamP->maxval, TGA_MAXVAL));
             putchar(pnm_scalesample(tuple[PAM_RED_PLANE], 
                                     pamP->maxval, TGA_MAXVAL));
-            if (pamP->depth > 3)
+            if (withAlpha)
                 putchar(pnm_scalesample(tuple[PAM_TRN_PLANE], 
                                         pamP->maxval, TGA_MAXVAL));
         }
@@ -291,9 +301,9 @@ computeOutName(struct cmdlineInfo const cmdline,
 static void
 validateTupleType(struct pam * const pamP) {
 
-    if (strcmp(pamP->tuple_type, "RGBA") == 0) {
+    if (strcmp(pamP->tuple_type, "RGB_ALPHA") == 0) {
         if (pamP->depth < 4)
-            pm_error("Invalid depth for tuple type RGBA.  "
+            pm_error("Invalid depth for tuple type RGB_ALPHA.  "
                      "Should have at least 4 planes, but has %d.", 
                      pamP->depth);
     } else if (strcmp(pamP->tuple_type, "RGB") == 0) {
@@ -305,7 +315,7 @@ validateTupleType(struct pam * const pamP) {
     } else if (strcmp(pamP->tuple_type, "BLACKANDWHITE") == 0) {
     } else 
         pm_error("Invalid type of input.  PAM tuple type is '%s'.  "
-                 "This programs understands only RGBA, RGB, GRAYSCALE, "
+                 "This programs understands only RGB_ALPHA, RGB, GRAYSCALE, "
                  "and BLACKANDWHITE.", pamP->tuple_type);
 }
 
@@ -316,18 +326,25 @@ computeImageType_cht(struct pam *            const pamP,
                      struct cmdlineInfo      const cmdline, 
                      tuple **                const tuples,
                      enum TGAbaseImageType * const baseImgTypeP,
+                     bool *                  const withAlphaP,
                      tupletable *            const chvP,
                      tuplehash *             const chtP, 
                      int *                   const ncolorsP) {
 
     unsigned int ncolors;
     enum TGAbaseImageType baseImgType;
+    bool withAlpha;
 
     validateTupleType(pamP);
 
+    withAlpha = (strcmp(pamP->tuple_type, "RGB_ALPHA") == 0);
+
     if (cmdline.defaultFormat) {
         /* default the image type */
-        if (pamP->depth > 1) {
+        if (withAlpha) {
+            baseImgType = TGA_RGB_TYPE;
+            *chvP = NULL;
+        } else if (pamP->depth > 1) {
             pm_message("computing colormap...");
             *chvP = 
                 pnm_computetuplefreqtable(pamP, tuples, MAXCOLORS, &ncolors);
@@ -336,12 +353,17 @@ computeImageType_cht(struct pam *            const pamP,
                 baseImgType = TGA_RGB_TYPE;
             } else 
                 baseImgType = TGA_MAP_TYPE;
-        } else
+        } else {
             baseImgType = TGA_MONO_TYPE;
+            *chvP = NULL;
+        }
     } else {
         baseImgType = cmdline.imgType;
 
         if (baseImgType == TGA_MAP_TYPE) {
+            if (withAlpha)
+                pm_error("Can't do a colormap because image has transparency "
+                         "information");
             pm_message("computing colormap...");
             *chvP = 
                 pnm_computetuplefreqtable(pamP, tuples, MAXCOLORS, &ncolors);
@@ -349,7 +371,8 @@ computeImageType_cht(struct pam *            const pamP,
                 pm_error("Too many colors for colormapped TGA.  "
                          "Use 'pnmquant %d' to reduce the number of colors.", 
                          MAXCOLORS);
-        }
+        } else
+            *chvP = NULL;
         if (baseImgType == TGA_MONO_TYPE && pamP->depth > 1)
             pm_error("For Mono TGA output, input must be "
                      "GRAYSCALE or BLACKANDWHITE PAM or PBM or PGM");
@@ -363,7 +386,8 @@ computeImageType_cht(struct pam *            const pamP,
         *chtP = NULL;
 
     *baseImgTypeP = baseImgType;
-    *ncolorsP = ncolors;
+    *withAlphaP   = withAlpha;
+    *ncolorsP     = ncolors;
 }
 
 
@@ -371,6 +395,7 @@ computeImageType_cht(struct pam *            const pamP,
 static void
 computeTgaHeader(struct pam *          const pamP,
                  enum TGAbaseImageType const baseImgType,
+                 bool                  const withAlpha,
                  bool                  const rle, 
                  int                   const ncolors,
                  unsigned char         const orgBit,
@@ -391,10 +416,11 @@ computeTgaHeader(struct pam *          const pamP,
         }
     }
     
-    if (id)
-        tgaHeaderP->IDLength = strlen(id);
-    else
-        tgaHeaderP->IDLength = 0;
+    if (id) {
+        tgaHeaderP->IdLength = strlen(id);
+        tgaHeaderP->Id = strdup(id);
+    } else
+        tgaHeaderP->IdLength = 0;
     tgaHeaderP->Index_lo = 0;
     tgaHeaderP->Index_hi = 0;
     if (baseImgType == TGA_MAP_TYPE) {
@@ -413,7 +439,7 @@ computeTgaHeader(struct pam *          const pamP,
         tgaHeaderP->PixelSize = 8;
         break;
     case TGA_RGB_TYPE:
-        tgaHeaderP->PixelSize = 8 * MAX(3, pamP->depth);
+        tgaHeaderP->PixelSize = 8 * MAX((withAlpha ? 4: 3), pamP->depth);
         break;
     case TGA_MONO_TYPE:
         tgaHeaderP->PixelSize = 8;
@@ -432,11 +458,20 @@ computeTgaHeader(struct pam *          const pamP,
 
 
 
+static void
+releaseTgaHeader(struct ImageHeader const tgaHeader) {
+
+    strfree(tgaHeader.Id);
+}
+
+
+
 static void 
 writeTgaRaster(struct pam *          const pamP,
                tuple **              const tuples, 
                tuplehash             const cht,
                enum TGAbaseImageType const imgType,
+               bool                  const withAlpha,
                bool                  const rle,
                unsigned char         const orgBit) {
 
@@ -444,7 +479,7 @@ writeTgaRaster(struct pam *          const pamP,
     int row;
 
     if (rle)
-        runlength = (int*) malloc(pamP->width * sizeof(int));
+        MALLOCARRAY(runlength, pamP->width);
 
     for (row = 0; row < pamP->height; ++row) {
         int realrow;
@@ -455,13 +490,15 @@ writeTgaRaster(struct pam *          const pamP,
             for (col = 0; col < pamP->width; ) {
                 if (runlength[col] > 0) {
                     putchar(0x80 + runlength[col] - 1);
-                    putPixel(pamP, tuples[realrow][col], imgType, cht);
+                    putPixel(pamP, tuples[realrow][col], imgType, withAlpha,
+                             cht);
                     col += runlength[col];
                 } else if (runlength[col] < 0) {
                     int i;
                     putchar(-runlength[col] - 1);
                     for (i = 0; i < -runlength[col]; ++i)
-                        putPixel(pamP, tuples[realrow][col+i], imgType, cht);
+                        putPixel(pamP, tuples[realrow][col+i], 
+                                 imgType, withAlpha, cht);
                     col += -runlength[col];
                 } else
                     pm_error("Internal error: zero run length");
@@ -469,7 +506,7 @@ writeTgaRaster(struct pam *          const pamP,
         } else {
             int col;
             for (col = 0; col < pamP->width; ++col)
-                putPixel(pamP, tuples[realrow][col], imgType, cht);
+                putPixel(pamP, tuples[realrow][col], imgType, withAlpha, cht);
         }
     }
     if (rle)
@@ -490,6 +527,7 @@ main(int argc, char *argv[]) {
     tuplehash cht;
     struct ImageHeader tgaHeader;
     enum TGAbaseImageType baseImgType;
+    bool withAlpha;
     const char *outName;
 
     pnm_init( &argc, argv );
@@ -500,14 +538,14 @@ main(int argc, char *argv[]) {
 
     computeOutName(cmdline, &outName);
 
-    tuples = pnm_readpam(ifP, &pam, sizeof(pam));
+    tuples = pnm_readpam(ifP, &pam, PAM_STRUCT_SIZE(tuple_type));
     pm_close(ifP);
 
     computeImageType_cht(&pam, cmdline, tuples, 
-                         &baseImgType, &chv, &cht, &ncolors);
+                         &baseImgType, &withAlpha, &chv, &cht, &ncolors);
 
     /* Do the Targa header */
-    computeTgaHeader(&pam, baseImgType, !cmdline.norle,
+    computeTgaHeader(&pam, baseImgType, withAlpha, !cmdline.norle,
                      ncolors, 0, outName, &tgaHeader);
     writeTgaHeader(tgaHeader);
     
@@ -518,14 +556,17 @@ main(int argc, char *argv[]) {
             putMapEntry(&pam, chv[i]->tuple, tgaHeader.CoSize);
     }
 
-    writeTgaRaster(&pam, tuples, cht, baseImgType, !cmdline.norle, 0);
+    writeTgaRaster(&pam, tuples, cht, baseImgType, withAlpha, 
+                   !cmdline.norle, 0);
 
     if (cht)
         pnm_destroytuplehash(cht);
     if (chv)
         pnm_freetupletable(&pam, chv);
 
+    releaseTgaHeader(tgaHeader);
+    strfree(outName);
     pnm_freepamarray(tuples, &pam);
 
-    exit(0);
+    return 0;
 }
