@@ -4,6 +4,9 @@
   compile with: cc -o pbmtopk pbmtopk.c -lm -lpbm
 */
 
+#define _BSD_SOURCE 1      /* Make sure strdup() is in string.h */
+#define _XOPEN_SOURCE 500  /* Make sure strdup() is in string.h */
+
 #include <stdio.h>
 #include <math.h>
 #include <ctype.h>
@@ -11,6 +14,7 @@
 
 #include "pbm.h"
 #include "nstring.h"
+#include "mallocvar.h"
 
 #define MAXPKCHAR 256
 #define MAXOPTLINE 200
@@ -174,8 +178,8 @@ pbmtopk_add_suffix(char * const name,
 
 
 /* initialize the PK parameters */
-static void initialize_pk()
-{
+static void 
+initialize_pk(void) {
     integer i ;
     pm_message("This is PBMtoPK, version 2.4") ;
     for (i = 127 ; i <= 255 ; i ++) xchr[i] = '?' ;
@@ -188,47 +192,64 @@ static void initialize_pk()
     designsize = fixword(1.0) ;
 }
 
+
+
 /* write a single byte to the PK file */
-static void pbmtopk_pkbyte(b)
-    integer b ;
-{
-    if (b < 0) b += 256 ;
+static void 
+pbmtopk_pkbyte(integer const b_in) {
+    integer b;
+
+    b = b_in;  /* initial value */
+
+    if (b < 0) 
+        b += 256 ;
     putc(b, pkfile) ;
     pbmtopk_pkloc++ ;
 }
 
+
+
 /* write two bytes to the PK file */
-static void pkhalfword(a)
-    integer a ;
-{
-    if (a < 0) a += 65536 ;
+static void 
+pkhalfword(integer const a_in) {
+    integer a;
+
+    a = a_in;
+
+    if (a < 0) 
+        a += 65536 ;
     pbmtopk_pkbyte(a >> 8) ;
     pbmtopk_pkbyte(a & 255) ;
 }
 
+
+
 /* write three bytes to the PK file */
-static void pkthreebytes(a)
-    integer a ;
-{
+static void 
+pkthreebytes(integer const a) {
+
     pbmtopk_pkbyte((a>>16) & 255) ;
     pbmtopk_pkbyte((a>>8) & 255) ;
     pbmtopk_pkbyte(a & 255) ;
 }
 
+
+
 /* write four bytes to the PK file */
-static void pkword(a)
-    integer a ;
-{
+static void 
+pkword(integer const a) {
     pbmtopk_pkbyte((a>>24) & 255) ;
     pbmtopk_pkbyte((a>>16) & 255) ;
     pbmtopk_pkbyte((a>>8) & 255) ;
     pbmtopk_pkbyte(a & 255) ;
 }
 
+
+
 /* write a nibble to the PK file */
-static void pknyb(a)
-    integer a ;
-{
+static void 
+pknyb(integer const a) {
+
     if (bitweight == 16) {
         outputbyte = (a<<4) ;
         bitweight = 1 ;
@@ -238,61 +259,80 @@ static void pknyb(a)
     }
 }
 
+
+
 /* write preamble to PK file */
-static void writepreamble()
-{
+static void 
+writepreamble(void) {
     integer i ;
     const char * const comment = "PBMtoPK 2.4 output" ;
    
     pbmtopk_pkbyte(247) ;                /* PRE command */
     pbmtopk_pkbyte(89) ;             /* PK file type */
     pbmtopk_pkbyte(strlen(comment)) ;            /* output comment */
-    for (i = 0 ; i < strlen(comment); i++) pbmtopk_pkbyte(xord[(int)comment[i]]) ;
+    for (i = 0 ; i < strlen(comment); i++) 
+        pbmtopk_pkbyte(xord[(int)comment[i]]) ;
     pkword(designsize) ;             /* write designsize */
     pkword(checksum) ;       /* write checksum; calculate if possible */
     pkword(hppp) ;               /* write H pixels per point */
     pkword(hppp) ;               /* write V pixels per point */
 }
 
+
+
 /* write postamble to PK file, padded to word length */
-static void writepostamble()
-{
+static void 
+writepostamble(void) {
     pbmtopk_pkbyte(245) ;                /* POST command */
     while (pbmtopk_pkloc % 4)
         pbmtopk_pkbyte(246) ;             /* pad with no-ops */
     pm_message("%d bytes written to packed file.", pbmtopk_pkloc) ;
 }
 
+
+
 /* write a byte to the TFM file */
-static void tfmbyte(b)
-    integer b ;
-{
+static void 
+tfmbyte(integer const b_in) {
+    integer b;
+
+    b = b_in;
+
     if (b < 0) b += 256 ;
     putc(b, tfmfile) ;
 }
 
+
+
 /* write a half word to the TFM file */
-static void tfmhalfword(a)
-    integer a ;
-{
+static void 
+tfmhalfword(integer const a_in) {
+    
+    integer a;
+
+    a = a_in;
+
     if (a < 0) a += 65536 ;
     tfmbyte(a >> 8) ;
     tfmbyte(a & 255) ;
 }
 
+
+
 /* write a word to the TFM file */
-static void tfmword(a)
-    integer a ;
-{
+static void 
+tfmword(integer const a) {
     tfmbyte((a>>24) & 255) ;
     tfmbyte((a>>16) & 255) ;
     tfmbyte((a>>8) & 255) ;
     tfmbyte(a & 255) ;
 }
 
+
+
 /* write the whole TFM file for the font */
-static void writetfmfile()
-{
+static void 
+writetfmfile(void) {
     integer totallength ;
     integer headersize = 17;
     integer i ;
@@ -336,16 +376,24 @@ static void writetfmfile()
     /* header */
     tfmword(checksum) ;              /* write checksum */
     tfmword(designsize) ;            /* write designsize */
-    if (strlen(codingscheme) > 39) tfmbyte(39) ; /* write coding scheme len */
-    else tfmbyte(strlen(codingscheme)) ;
+    if (strlen(codingscheme) > 39) 
+        tfmbyte(39) ; /* write coding scheme len */
+    else 
+        tfmbyte(strlen(codingscheme)) ;
     for (i = 0; i < 39; i++)         /* write coding scheme */
-        if (*codingscheme) tfmbyte(xord[(int)(*codingscheme++)]) ;
-        else tfmbyte(0) ;
-    if (strlen(familyname) > 19) tfmbyte(19) ;   /* write family length */
-    else tfmbyte(strlen(familyname)) ;
+        if 
+            (*codingscheme) tfmbyte(xord[(int)(*codingscheme++)]) ;
+        else
+            tfmbyte(0) ;
+    if (strlen(familyname) > 19) 
+        tfmbyte(19) ;   /* write family length */
+    else 
+        tfmbyte(strlen(familyname)) ;
     for (i = 0; i < 19; i++)         /* write family */
-        if (*familyname) tfmbyte(xord[(int)(*familyname++)]) ;
-        else tfmbyte(0) ;
+        if (*familyname) 
+            tfmbyte(xord[(int)(*familyname++)]) ;
+        else 
+            tfmbyte(0) ;
     /* char_info */
     for (car = smallestch; car <= largestch; car++)
         if (filename[car]) {          /* write character info */
@@ -353,7 +401,8 @@ static void writetfmfile()
             tfmbyte((hgtindex[car]<<4) + depindex[car]) ;
             tfmbyte(italindex[car]<<2) ;
             tfmbyte(0) ;
-        } else tfmword(0) ;
+        } else
+            tfmword(0) ;
     /* width table */
     for (i = 0; i < numwidth; i++) tfmword(widthtab[i]) ;
     /* height table */
@@ -372,9 +421,10 @@ static void writetfmfile()
     pm_message("%d bytes written to tfm file.", totallength*4) ;
 }
 
+
+
 /* read a character from a PBM file */
-static void readcharacter()
-{
+static void readcharacter(void) {
     FILE *fp;
    
     fp = pm_openr(filename[car]);
@@ -390,7 +440,8 @@ static void readcharacter()
     if ((charflags[car] & TFMHEIGHT) == 0)
         hgtindex[car] = add_tfmheight(fixword(designunits(yoffset[car]+1)));
     if ((charflags[car] & TFMDEPTH) == 0)
-        depindex[car] = add_tfmdepth(fixword(designunits(height-1-yoffset[car])));
+        depindex[car] = 
+            add_tfmdepth(fixword(designunits(height-1-yoffset[car])));
     if ((charflags[car] & TFMITALIC) == 0) italindex[car] = 0;
    
     if (car < smallestch) smallestch = car;
@@ -398,19 +449,27 @@ static void readcharacter()
     if (width > emwidth) emwidth = width ;
 }
 
+
+
 /* test if two rows of the PBM are the same */
-static int equal(row1, row2)
-    bit *row1, *row2 ;
-{
+static int 
+equal(const bit * const row1, 
+      const bit * const row2) {
+
     integer i ;
    
     for (i = 0; i < width; i++)
-        if (row1[i] != row2[i]) return (0) ;
+        if (row1[i] != row2[i]) 
+            return (0) ;
+
     return(1) ;
 }
 
-static void shipcharacter()
-{
+
+
+static void 
+shipcharacter(void) {
+
     integer compsize ;
     integer i, j, k ;
     bit *zerorow, *onesrow ;
@@ -444,10 +503,8 @@ static void shipcharacter()
     integer xoff = xoffset[car] ;
     integer yoff = yoffset[car] ;
    
-    repeatptr =
-        (integer *)malloc((unsigned int)((height+1)*sizeof(integer))) ;
-    bitcounts =
-        (integer *)malloc((unsigned int)((height*width)*sizeof(integer))) ;
+    MALLOCARRAY(repeatptr, height + 1);
+    MALLOCARRAY(bitcounts, height * width);
     if (repeatptr == NULL || bitcounts == NULL)
         pm_error("out of memory while allocating bit counts");
     zerorow = pbm_allocrow(width) ;      /* initialise plain rows */
@@ -700,13 +757,17 @@ static void shipcharacter()
     free((char *)bitcounts);
 }
 
+
+
 /* check that character is in valid range */
-static void checkchar()
-{
+static void 
+checkchar(void) {
     if (car < 0 || car >= MAXPKCHAR)
         pm_error("character must be in range 0 to %d", MAXPKCHAR-1) ;
 }
 
+
+
 /* read character information from an option file */
 static void 
 optionfile(const char * const name) {
@@ -730,11 +791,9 @@ optionfile(const char * const name) {
             checkchar() ;
             if (sscanf(here, "%s%n", str, &n) != 1)
                 pm_error("bad option file line %s", buffer) ;
-            filename[car] =
-                (char *)malloc((unsigned int)(sizeof(char)*(strlen(str)+1))) ;
+            filename[car] = strdup(str);
             if (filename[car] == NULL)
                 pm_error("out of memory allocating filename %s", str);
-            strcpy(filename[car], str) ;
             for (i = 1; i < 256; i<<=1) {
                 here += n;
                 if (sscanf(here, "%s%n", str, &n) != 1) break ;
@@ -777,19 +836,18 @@ optionfile(const char * const name) {
 
 
 int
-main(argc, argv)
-    int argc ;
-    char *argv[] ;
-{
+main(int argc, char *argv[]) {
     integer i, hesc, vesc, xoff, yoff, tfwid, tfdep, tfhgt, tfital ;
     byte flags ;
-    const char * const usage = "pkfile[.pk] tfmfile[.tfm] dpi [-s designsize] [-p num param...]\n\
-[-C codingscheme ] [-F family] [-c num | <char>]...\n\
-<char> is:\n\
-[-W tfmwidth] [-H tfmheight] [-D tfmdepth] [-I ital_corr] [-h horiz]\n\
-[-v vert] [-x xoffset] [-y yoffset] file\n\
-or:\n\
--f optfile\n" ;
+    const char * const usage = "pkfile[.pk] tfmfile[.tfm] dpi "
+        "[-s designsize] [-p num param...]\n"
+        "[-C codingscheme ] [-F family] [-c num | <char>]...\n"
+        "<char> is:\n"
+        "[-W tfmwidth] [-H tfmheight] [-D tfmdepth] [-I ital_corr] "
+        "[-h horiz]\n"
+        "[-v vert] [-x xoffset] [-y yoffset] file\n"
+        "or:\n"
+        "-f optfile\n" ;
 
     pbm_init(&argc, argv);
     initialize_pk() ;
@@ -911,6 +969,7 @@ or:\n\
     writetfmfile() ;
     pm_close(pkfile) ;
     pm_close(tfmfile) ;
-    exit(0);
+
+    return 0;
 }
 
