@@ -1,6 +1,4 @@
-/*\
- * $Id: ppmtobmp.c,v 1.9 1992/11/24 19:39:33 dws Exp dws $
- *
+/*
  * ppmtobmp.c - Converts from a PPM file to a Microsoft Windows or OS/2
  * .BMP file.
  *
@@ -13,36 +11,12 @@
  * in supporting documentation.  This software is provided "as is"
  * without express or implied warranty.
  *
- * $Log: ppmtobmp.c,v $
- * Revision 1.9  1992/11/24  19:39:33  dws
- * Added copyright.
- *
- * Revision 1.8  1992/11/17  02:16:52  dws
- * Moved length functions to bmp.h.
- *
- * Revision 1.7  1992/11/11  23:18:16  dws
- * Modified to adjust the bits per pixel to 1, 4, or 8.
- *
- * Revision 1.6  1992/11/11  22:43:39  dws
- * Commented out a superfluous message.
- *
- * Revision 1.5  1992/11/11  05:58:06  dws
- * First version that works.
- *
- * Revision 1.4  1992/11/11  03:40:32  dws
- * Moved calculation of bits per pixel to BMPEncode.
- *
- * Revision 1.3  1992/11/11  03:02:34  dws
- * Added BMPEncode function.
- *
- * Revision 1.2  1992/11/08  01:44:35  dws
- * Added option processing and reading of PPM file.
- *
- * Revision 1.1  1992/11/08  00:46:07  dws
- * Initial revision
-\*/
+ */
 
 #define _BSD_SOURCE 1    /* Make sure strdup() is in string.h */
+#define _XOPEN_SOURCE 500  /* Make sure strdup() is in string.h */
+
+#include <assert.h>
 #include <string.h>
 #include "bmp.h"
 #include "ppm.h"
@@ -162,196 +136,197 @@ PutLong(FILE * const fp, long const v) {
  * BMP writing
  */
 
-/*
- * returns the number of bytes written, or -1 on error.
- */
 static int
-BMPwritefileheader(fp, class, bitcount, x, y)
-        FILE           *fp;
-        int             class;
-        unsigned long   bitcount;
-        unsigned long   x;
-        unsigned long   y;
-{
-        PutByte(fp, 'B');
-        PutByte(fp, 'M');
+BMPwritefileheader(FILE *        const fp, 
+                   int           const class, 
+                   unsigned long const bitcount, 
+                   unsigned long const x, 
+                   unsigned long const y) {
+/*----------------------------------------------------------------------------
+  Return the number of bytes written, or -1 on error.
+-----------------------------------------------------------------------------*/
+    PutByte(fp, 'B');
+    PutByte(fp, 'M');
 
-        /* cbSize */
-        PutLong(fp, BMPlenfile(class, bitcount, -1, x, y));
-
-        /* xHotSpot */
-        PutShort(fp, 0);
-
-        /* yHotSpot */
-        PutShort(fp, 0);
-
-        /* offBits */
-        PutLong(fp, BMPoffbits(class, bitcount, -1));
-
-        return 14;
+    /* cbSize */
+    PutLong(fp, BMPlenfile(class, bitcount, -1, x, y));
+    
+    /* xHotSpot */
+    PutShort(fp, 0);
+    
+    /* yHotSpot */
+    PutShort(fp, 0);
+    
+    /* offBits */
+    PutLong(fp, BMPoffbits(class, bitcount, -1));
+    
+    return 14;
 }
 
-/*
- * returns the number of bytes written, or -1 on error.
- */
+
+
 static int
-BMPwriteinfoheader(fp, class, bitcount, x, y)
-        FILE           *fp;
-        int             class;
-        unsigned long   bitcount;
-        unsigned long   x;
-        unsigned long   y;
-{
-        long    cbFix;
+BMPwriteinfoheader(FILE *        const fp, 
+                   int           const class, 
+                   unsigned long const bitcount, 
+                   unsigned long const x, 
+                   unsigned long const y) {
+/*----------------------------------------------------------------------------
+  Return the number of bytes written, or -1 on error.
+----------------------------------------------------------------------------*/
+    long cbFix;
 
-        /* cbFix */
-        switch (class)
-        {
-        case C_WIN:
-                cbFix = 40;
-                PutLong(fp, cbFix);
+    switch (class) {
+    case C_WIN: {
+        cbFix = 40;
+        PutLong(fp, cbFix);
 
-                /* cx */
-                PutLong(fp, x);
-                /* cy */
-                PutLong(fp, y);
-                /* cPlanes */
-                PutShort(fp, 1);
-                /* cBitCount */
-                PutShort(fp, bitcount);
-
-                /*
-                 * We've written 16 bytes so far, need to write 24 more
-                 * for the required total of 40.
-                 */
-
-                PutLong(fp, 0);
-                PutLong(fp, 0);
-                PutLong(fp, 0);
-                PutLong(fp, 0);
-                PutLong(fp, 0);
-                PutLong(fp, 0);
-
-
-                break;
-        case C_OS2:
-                cbFix = 12;
-                PutLong(fp, cbFix);
-
-                /* cx */
-                PutShort(fp, x);
-                /* cy */
-                PutShort(fp, y);
-                /* cPlanes */
-                PutShort(fp, 1);
-                /* cBitCount */
-                PutShort(fp, bitcount);
-
-                break;
-        default:
-                pm_error(er_internal, "BMPwriteinfoheader");
-        }
-
-        return cbFix;
-}
-
-/*
- * returns the number of bytes written, or -1 on error.
- */
-static int
-BMPwritergb(FILE *fp, int class, pixval R, pixval G, pixval B)
-{
-        switch (class)
-        {
-        case C_WIN:
-                PutByte(fp, B);
-                PutByte(fp, G);
-                PutByte(fp, R);
-                PutByte(fp, 0);
-                return 4;
-        case C_OS2:
-                PutByte(fp, B);
-                PutByte(fp, G);
-                PutByte(fp, R);
-                return 3;
-        default:
-                pm_error(er_internal, "BMPwritergb");
-        }
-        return -1;
-}
-
-/*
- * returns the number of bytes written, or -1 on error.
- */
-static int
-BMPwritecolormap(FILE *fp, const int class, const int bpp,
-                 const int colors, 
-                 const unsigned char R[], const unsigned char G[], 
-                 const unsigned char B[])
-{
-        int             nbyte = 0;
-        int             i;
-        long            ncolors;
-
-        for (i = 0; i < colors; i++)
-        {
-                nbyte += BMPwritergb(fp,class,R[i],G[i],B[i]);
-        }
-
-        ncolors = (1 << bpp);
-
-        for (; i < ncolors; i++)
-        {
-                nbyte += BMPwritergb(fp,class,0,0,0);
-        }
-
-        return nbyte;
-}
-
-/*
- * returns the number of bytes written, or -1 on error.
- */
-static int
-BMPwriterow_palette(FILE *fp, const pixel * const row, 
-                    const unsigned long cx, 
-                    const unsigned short bpp, const colorhash_table cht)
-{
-        BITSTREAM       b;
-        unsigned        nbyte = 0;
-        int             rc;
-        unsigned        x;
-
-        if ((b = pm_bitinit(fp, "w")) == (BITSTREAM) 0)
-        {
-                return -1;
-        }
-
-        for (x = 0; x < cx; x++)
-        {
-            if ((rc = pm_bitwrite(b, bpp, 
-                                  ppm_lookupcolor(cht, &row[x]))) == -1)
-            {
-                return -1;
-            }
-            nbyte += rc;
-        }
-
-        if ((rc = pm_bitfini(b)) == -1)
-        {
-                return -1;
-        }
-        nbyte += rc;
+        PutLong(fp, x);         /* cx */
+        PutLong(fp, y);         /* cy */
+        PutShort(fp, 1);        /* cPlanes */
+        PutShort(fp, bitcount); /* cBitCount */
 
         /*
-         * Make sure we write a multiple of 4 bytes.
+         * We've written 16 bytes so far, need to write 24 more
+         * for the required total of 40.
          */
-        while (nbyte % 4)
-        {
-                PutByte(fp, 0);
-                nbyte++;
-        }
 
-        return nbyte;
+        PutLong(fp, 0);   /* Compression */
+        PutLong(fp, 0);   /* ImageSize */
+        PutLong(fp, 0);   /* XpixelsPerMeter */
+        PutLong(fp, 0);   /* YpixelsPerMeter */
+        PutLong(fp, 0);   /* ColorsUsed */
+        PutLong(fp, 0);   /* ColorsImportant */
+    }
+    break;
+    case C_OS2: {
+        cbFix = 12;
+        PutLong(fp, cbFix);
+
+        PutShort(fp, x);        /* cx */
+        PutShort(fp, y);        /* cy */
+        PutShort(fp, 1);        /* cPlanes */
+        PutShort(fp, bitcount); /* cBitCount */
+    }
+    break;
+    default:
+        pm_error(er_internal, "BMPwriteinfoheader");
+    }
+
+    return cbFix;
+}
+
+
+
+static int
+BMPwritergb(FILE * const fp, 
+            int    const class, 
+            pixval const R, 
+            pixval const G, 
+            pixval const B) {
+/*----------------------------------------------------------------------------
+  Return the number of bytes written, or -1 on error.
+-----------------------------------------------------------------------------*/
+    switch (class) {
+    case C_WIN:
+        PutByte(fp, B);
+        PutByte(fp, G);
+        PutByte(fp, R);
+        PutByte(fp, 0);
+        return 4;
+    case C_OS2:
+        PutByte(fp, B);
+        PutByte(fp, G);
+        PutByte(fp, R);
+        return 3;
+    default:
+        pm_error(er_internal, "BMPwritergb");
+    }
+    return -1;
+}
+
+
+
+static int
+BMPwritecolormap(FILE *        const fp, 
+                 int           const class, 
+                 int           const bpp,
+                 int           const colors, 
+                 unsigned char const R[], 
+                 unsigned char const G[], 
+                 unsigned char const B[]) {
+/*----------------------------------------------------------------------------
+  Return the number of bytes written, or -1 on error.
+-----------------------------------------------------------------------------*/
+    long const ncolors = (1 << bpp);
+
+    int  nbyte;
+    int  i;
+
+    nbyte = 0;
+    for (i = 0; i < colors; ++i)
+        nbyte += BMPwritergb(fp,class,R[i],G[i],B[i]);
+
+    for (; i < ncolors; ++i)
+        nbyte += BMPwritergb(fp,class,0,0,0);
+
+    return nbyte;
+}
+
+
+
+static int
+BMPwriterow_palette(FILE *          const fp, 
+                    const pixel *   const row, 
+                    unsigned long   const cx, 
+                    unsigned short  const bpp, 
+                    colorhash_table const cht) {
+/*----------------------------------------------------------------------------
+  Return the number of bytes written, or -1 on error.
+-----------------------------------------------------------------------------*/
+    BITSTREAM    b;
+    int retval;
+    
+    b = pm_bitinit(fp, "w");
+    if (b == NULL)
+        retval = -1;
+    else {
+        unsigned int nbyte;
+        unsigned int x;
+        bool         error;
+        
+        nbyte = 0;      /* initial value */
+        error = FALSE;  /* initial value */
+        
+        for (x = 0; x < cx && !error; ++x) {
+            int rc;
+            rc = pm_bitwrite(b, bpp, ppm_lookupcolor(cht, &row[x]));
+            if (rc == -1)
+                error = TRUE;
+            else
+                nbyte += rc;
+        }
+        if (error)
+            retval = -1;
+        else {
+            int rc;
+
+            rc = pm_bitfini(b);
+            if (rc == -1)
+                retval = -1;
+            else {
+                nbyte += rc;
+                
+                /* Make sure we write a multiple of 4 bytes.  */
+                while (nbyte % 4 != 0) {
+                    PutByte(fp, 0);
+                    ++nbyte;
+                }
+                retval = nbyte;
+            }
+        }
+    }
+    return retval;
 }
 
 
@@ -400,9 +375,8 @@ BMPwriterow_truecolor(FILE *        const fp,
     return nbyte;
 }
 
-/*
- * returns the number of bytes written, or -1 on error.
- */
+
+
 static int
 BMPwritebits(FILE *          const fp, 
              unsigned long   const cx, 
@@ -411,52 +385,40 @@ BMPwritebits(FILE *          const fp,
              unsigned short  const cBitCount, 
              const pixel **  const pixels, 
              pixval          const maxval,
-             colorhash_table const cht)
-{
-        int             nbyte = 0;
-        long            y;
+             colorhash_table const cht) {
+/*----------------------------------------------------------------------------
+  Return the number of bytes written, or -1 on error.
+-----------------------------------------------------------------------------*/
+    int  nbyte;
+    long y;
 
-        if(cBitCount > 24)
-        {
-                pm_error("cannot handle cBitCount: %d"
-                         ,cBitCount);
-        }
+    if (cBitCount > 24)
+        pm_error("cannot handle cBitCount: %d", cBitCount);
 
-        /*
-         * The picture is stored bottom line first, top line last
-         */
+    nbyte = 0;  /* initial value */
 
-        for (y = cy - 1; y >= 0; y--)
-        {
-                int rc;
-                if (colortype == PALETTE)
-                    rc = BMPwriterow_palette(fp, pixels[y], cx, 
-                                             cBitCount, cht);
-                else 
-                    rc = BMPwriterow_truecolor(fp, pixels[y], cx, maxval);
+    /* The picture is stored bottom line first, top line last */
 
-                if(rc == -1)
-                {
-                        pm_error("couldn't write row %ld", y);
-                }
-                if(rc%4)
-                {
-                        pm_error("row had bad number of bytes: %d"
-                                 ,rc);
-                }
-                nbyte += rc;
-        }
+    for (y = cy - 1; y >= 0; --y) {
+        int rc;
+        if (colortype == PALETTE)
+            rc = BMPwriterow_palette(fp, pixels[y], cx, 
+                                     cBitCount, cht);
+        else 
+            rc = BMPwriterow_truecolor(fp, pixels[y], cx, maxval);
 
-        return nbyte;
+        if (rc == -1)
+            pm_error("couldn't write row %ld", y);
+        if (rc % 4 != 0)
+            pm_error("row had bad number of bytes: %d", rc);
+        nbyte += rc;
+    }
+
+    return nbyte;
 }
 
-/*
- * Write a BMP file of the given class.
- *
- * Note that we must have 'colors' in order to know exactly how many
- * colors are in the R, G, B, arrays.  Entries beyond those in the
- * arrays are undefined.
- */
+
+
 static void
 BMPEncode(FILE *          const fp, 
           int             const class, 
@@ -470,46 +432,50 @@ BMPEncode(FILE *          const fp,
           colorhash_table const cht, 
           unsigned char   const R[], 
           unsigned char   const G[], 
-          unsigned char   const B[])
-{
-        unsigned long   nbyte = 0;
+          unsigned char   const B[]) {
+/*----------------------------------------------------------------------------
+  Write a BMP file of the given class.
+  
+  Note that we must have 'colors' in order to know exactly how many
+  colors are in the R, G, B, arrays.  Entries beyond those in the
+  arrays are undefined.
+-----------------------------------------------------------------------------*/
+    unsigned long nbyte;
 
-        if (colortype == PALETTE)
-            pm_message("Writing %d bits per pixel with a color palette", bpp);
-        else
-            pm_message("Writing %d bits per pixel truecolor (no palette)", 
-                       bpp);
+    if (colortype == PALETTE)
+        pm_message("Writing %d bits per pixel with a color palette", bpp);
+    else
+        pm_message("Writing %d bits per pixel truecolor (no palette)", bpp);
 
-        nbyte += BMPwritefileheader(fp, class, bpp, x, y);
-        nbyte += BMPwriteinfoheader(fp, class, bpp, x, y);
-        if (colortype == PALETTE)
-            nbyte += BMPwritecolormap(fp, class, bpp, colors, R, G, B);
+    nbyte = 0;  /* initial value */
+    nbyte += BMPwritefileheader(fp, class, bpp, x, y);
+    nbyte += BMPwriteinfoheader(fp, class, bpp, x, y);
+    if (colortype == PALETTE)
+        nbyte += BMPwritecolormap(fp, class, bpp, colors, R, G, B);
 
-        if(nbyte !=     ( BMPlenfileheader(class)
-                        + BMPleninfoheader(class)
-                        + BMPlencolormap(class, bpp, -1)))
-        {
-                pm_error(er_internal, "BMPEncode 1");
-        }
+    if (nbyte != (BMPlenfileheader(class)
+                  + BMPleninfoheader(class)
+                  + BMPlencolormap(class, bpp, -1)))
+        pm_error(er_internal, "BMPEncode 1");
 
-        nbyte += BMPwritebits(fp, x, y, colortype, bpp, pixels, maxval, cht);
-        if(nbyte != BMPlenfile(class, bpp, -1, x, y))
-        {
-                pm_error(er_internal, "BMPEncode 2");
-        }
+    nbyte += BMPwritebits(fp, x, y, colortype, bpp, pixels, maxval, cht);
+    if (nbyte != BMPlenfile(class, bpp, -1, x, y))
+        pm_error(er_internal, "BMPEncode 2");
 }
 
 
 
 static void
-analyze_colors(const pixel ** const pixels, 
-               const int cols, const int rows, const pixval maxval, 
-               int * const colors_p, 
-               int * const minimum_bpp_p,
-               colorhash_table *cht_p, 
-               unsigned char Red[], 
-               unsigned char Green[], 
-               unsigned char Blue[]) {
+analyze_colors(const pixel **    const pixels, 
+               int               const cols, 
+               int               const rows, 
+               pixval            const maxval, 
+               int *             const colors_p, 
+               int *             const minimum_bpp_p,
+               colorhash_table * const cht_p, 
+               unsigned char           Red[], 
+               unsigned char           Green[], 
+               unsigned char           Blue[]) {
 /*----------------------------------------------------------------------------
   Look at the colors in the image 'pixels' and compute values to use in
   representing those colors in a BMP image.  
@@ -527,7 +493,6 @@ analyze_colors(const pixel ** const pixels,
   If there are too many colors for a palette, return *cht_p == NULL.
 
   Issue informational messages.
-
 -----------------------------------------------------------------------------*/
 
     /* Figure out the colormap. */
@@ -542,13 +507,29 @@ analyze_colors(const pixel ** const pixels,
         *minimum_bpp_p = 24;
         *cht_p = NULL;
     } else {
-        *minimum_bpp_p = pm_maxvaltobits(*colors_p-1);
+        unsigned int const minbits = pm_maxvaltobits(*colors_p-1);
+
         pm_message("%d colors found", *colors_p);
+
+        /* Only 1, 4, 8, and 24 are defined in the BMP spec we
+           implement and other bpp's have in fact been seen to confuse
+           viewers.  There is an extended BMP format that has 16 bpp
+           too, but this program doesn't know how to generate that
+           (see Bmptopnm.c, though).  
+        */
+        if (minbits == 1)
+            *minimum_bpp_p = 1;
+        else if (minbits <= 4)
+            *minimum_bpp_p = 4;
+        else if (minbits <= 8)
+            *minimum_bpp_p = 8;
+        else
+            *minimum_bpp_p = 24;
+
         /*
          * Now scale the maxval to 255 as required by BMP format.
          */
-        for (i = 0; i < *colors_p; ++i)
-        {
+        for (i = 0; i < *colors_p; ++i) {
             Red[i] = (pixval) PPM_GETR(chv[i].color) * 255 / maxval;
             Green[i] = (pixval) PPM_GETG(chv[i].color) * 255 / maxval;
             Blue[i] = (pixval) PPM_GETB(chv[i].color) * 255 / maxval;
@@ -563,13 +544,16 @@ analyze_colors(const pixel ** const pixels,
 
 
 static void
-choose_colortype_bpp(const struct cmdline_info cmdline,
-                     const int colors, const int minimum_bpp,
-                     enum colortype * const colortype_p, 
-                     int * const bits_per_pixel_p) {
+choose_colortype_bpp(struct cmdline_info const cmdline,
+                     int                 const colors, 
+                     int                 const minimum_bpp,
+                     enum colortype *    const colortype_p, 
+                     int *               const bits_per_pixel_p) {
 
     if (!cmdline.bppSpec) {
-        /* User has no preference as to bits per pixel. */
+        /* User has no preference as to bits per pixel.  Choose the
+           smallest number possible for this image.
+        */
         *bits_per_pixel_p = minimum_bpp;
     } else {
         if (cmdline.bpp < minimum_bpp)
@@ -582,62 +566,65 @@ choose_colortype_bpp(const struct cmdline_info cmdline,
         else
             *bits_per_pixel_p = cmdline.bpp;
     }
+
+    assert(*bits_per_pixel_p == 1 || 
+           *bits_per_pixel_p == 4 || 
+           *bits_per_pixel_p == 8 || 
+           *bits_per_pixel_p == 24);
+
     if (*bits_per_pixel_p > 8) 
         *colortype_p = TRUECOLOR;
-    else
+    else {
         *colortype_p = PALETTE;
+    }
 }
 
 
 
 int
-main(int argc, char **argv)
-{
-        FILE *ifp;
+main(int argc, char **argv) {
+    FILE *ifp;
 
-        int rows;
-        int cols;
-        int colors;
-        pixval maxval;
-        int minimum_bpp;
-        int bits_per_pixel;
-        enum colortype colortype;
+    int rows;
+    int cols;
+    int colors;
+    pixval maxval;
+    int minimum_bpp;
+    int bits_per_pixel;
+    enum colortype colortype;
 
-        pixel** pixels;
-        colorhash_table cht;
-        /* 'Red', 'Green', and 'Blue' are the color map for the BMP file,
-           with indices the same as in 'cht', above.
-        */
-        unsigned char Red[MAXCOLORS], Green[MAXCOLORS], Blue[MAXCOLORS];
+    pixel** pixels;
+    colorhash_table cht;
+    /* 'Red', 'Green', and 'Blue' are the color map for the BMP file,
+       with indices the same as in 'cht', above.
+    */
+    unsigned char Red[MAXCOLORS], Green[MAXCOLORS], Blue[MAXCOLORS];
 
-        ppm_init(&argc, argv);
+    ppm_init(&argc, argv);
 
-        parse_command_line(argc, argv, &cmdline);
+    parse_command_line(argc, argv, &cmdline);
 
-        ifp = pm_openr(cmdline.input_filename);
+    ifp = pm_openr(cmdline.input_filename);
 
-        pixels = ppm_readppm(ifp, &cols, &rows, &maxval);
+    pixels = ppm_readppm(ifp, &cols, &rows, &maxval);
 
-        pm_close(ifp);
+    pm_close(ifp);
 
 
-        analyze_colors((const pixel**)pixels, cols, rows, maxval, 
-                       &colors, &minimum_bpp, &cht, Red, Green, Blue);
+    analyze_colors((const pixel**)pixels, cols, rows, maxval, 
+                   &colors, &minimum_bpp, &cht, Red, Green, Blue);
 
-        choose_colortype_bpp(cmdline, colors, minimum_bpp, &colortype, 
-                             &bits_per_pixel);
+    choose_colortype_bpp(cmdline, colors, minimum_bpp, &colortype, 
+                         &bits_per_pixel);
 
-        BMPEncode(stdout, cmdline.class, colortype, bits_per_pixel,
-                  cols, rows, (const pixel**)pixels, maxval, colors, cht,
-                  Red, Green, Blue);
+    BMPEncode(stdout, cmdline.class, colortype, bits_per_pixel,
+              cols, rows, (const pixel**)pixels, maxval, colors, cht,
+              Red, Green, Blue);
 
-        if (cht) 
-            ppm_freecolorhash(cht);
+    if (cht) 
+        ppm_freecolorhash(cht);
 
-        pm_close(stdout);
+    pm_close(stdout);
 
-        exit(0);
+    return 0;
 }
-
-
-
