@@ -68,7 +68,11 @@ parseCommandLine(int argc, char ** argv,
 
     if (argc-1 == 0) {
         if (cmdlineP->transferFunction == XF_SRGBRAMP)
-            cmdlineP->rgamma = cmdlineP->ggamma = cmdlineP->bgamma = 2.2;
+            /* The whole function is often approximated with exponent 2.2 and
+               no linear piece.  We do the linear piece, so we use the real
+               exponent of 2.4.
+            */
+            cmdlineP->rgamma = cmdlineP->ggamma = cmdlineP->bgamma = 2.4;
         else
             cmdlineP->rgamma = cmdlineP->ggamma = cmdlineP->bgamma = 1.0/0.45;
         cmdlineP->filespec = "-";
@@ -150,7 +154,7 @@ buildCie709Gamma(xelval table[], xelval const maxval,
         (1.099 * pow(0.018, oneOverGamma) - 0.099) / 0.018;
 
     for (i = 0; i <= linearCutoff; ++i) 
-        table[i] = i * linearExpansion;
+        table[i] = i * linearExpansion + 0.5;
     for (; i <= maxval; ++i) {
         double const normalized = ((double) i) / maxval;
             /* Xel sample value normalized to 0..1 */
@@ -179,12 +183,14 @@ buildCie709GammaInverse(xelval table[], xelval const maxval,
        The exponential is slightly stretched and translated, though,
        unlike the popular pure exponential gamma transfer function.
     */
+
     xelval const linearCutoff = (xelval) (maxval * 0.018 + 0.5);
     double const linearExpansion = 
         (1.099 * pow(0.018, oneOverGamma) - 0.099) / 0.018;
 
     for (i = 0; i <= linearCutoff * linearExpansion; ++i) 
-        table[i] = i / linearExpansion;
+        table[i] = i / linearExpansion + 0.5;
+
     for (; i <= maxval; ++i) {
         double const normalized = ((double) i) / maxval;
             /* Xel sample value normalized to 0..1 */
@@ -213,12 +219,12 @@ buildSrgbGamma(xelval table[], xelval const maxval,
        The exponential is slightly stretched and translated, though,
        unlike the popular pure exponential gamma transfer function.
     */
-    xelval const linearCutoff = (xelval) maxval * 0.040405 + 0.5;
+    xelval const linearCutoff = (xelval) maxval * 0.0031308 + 0.5;
     double const linearExpansion = 
-        (1.055 * pow(0.040405, oneOverGamma) - 0.055) / 0.040405;
+        (1.055 * pow(0.0031308, oneOverGamma) - 0.055) / 0.0031308;
 
     for (i = 0; i <= linearCutoff; ++i) 
-        table[i] = i * linearExpansion;
+        table[i] = i * linearExpansion + 0.5;
     for (; i <= maxval; ++i) {
         double const normalized = ((double) i) / maxval;
             /* Xel sample value normalized to 0..1 */
@@ -247,12 +253,12 @@ buildSrgbGammaInverse(xelval table[], xelval const maxval,
        The exponential is slightly stretched and translated, though,
        unlike the popular pure exponential gamma transfer function.
     */
-    xelval const linearCutoff = (xelval) maxval * 0.040405 + 0.5;
+    xelval const linearCutoff = (xelval) maxval * 0.0031308 + 0.5;
     double const linearExpansion = 
-        (1.055 * pow(0.040405, oneOverGamma) - 0.055) / 0.040405;
+        (1.055 * pow(0.0031308, oneOverGamma) - 0.055) / 0.0031308;
 
     for (i = 0; i <= linearCutoff * linearExpansion; ++i) 
-        table[i] = i / linearExpansion;
+        table[i] = i / linearExpansion + 0.5;
     for (; i <= maxval; ++i) {
         double const normalized = ((double) i) / maxval;
             /* Xel sample value normalized to 0..1 */
