@@ -309,7 +309,7 @@ atfilt_setup(double alpha, double radius, double maxscale) {
 
     /* do setup */
     
-    if (alpha >= 0.0 && alpha < 1.0) {
+    if (alpha >= 0.0 && alpha < 0.5) {
         /* alpha trimmed mean */
         double noinmean;
             /* number of elements (out of a possible 7) used in the mean */
@@ -333,7 +333,7 @@ atfilt_setup(double alpha, double radius, double maxscale) {
             alpharange = 3;
             alphafraction = (3.0 - noinmean)/2.0;
         }
-    } else if (alpha > 0.5) {
+    } else if (alpha >= 1.0 && alpha <= 2.0) {
         /* optimal estimation - alpha controls noise variance threshold. */
         int i;
         double noinmean = 7.0;
