@@ -618,11 +618,6 @@ pnm_readpaminit(FILE *       const file,
     /* Get magic number. */
     pamP->format = pm_readmagicnumber(file);
 
-    pamP->plainformat =
-        (pamP->format == PBM_FORMAT || 
-         pamP->format == PGM_FORMAT ||
-         pamP->format == PPM_FORMAT);
-
     switch (PAM_FORMAT_TYPE(pamP->format)) {
     case PAM_TYPE: 
         readpaminitrest(pamP);
@@ -657,6 +652,7 @@ pnm_readpaminit(FILE *       const file,
     }
     
     pamP->bytes_per_sample = pnm_bytespersample(pamP->maxval);
+    pamP->plainformat = FALSE;
 
     validateComputableSize(pamP);
 }
@@ -883,7 +879,7 @@ pm_feed_from_pamtuples(int    const pipeToFeedFd,
     outpam.file = fdopen(pipeToFeedFd, "w");
 
     /* The following signals (and normally kills) the process with
-       SIGPIPE if the pipe does not take all 'inputLength' bytes.
+       SIGPIPE if the pipe does not take all the data.
     */
     pnm_writepam(&outpam, *inputTuplesP->tuplesP);
 
@@ -898,10 +894,11 @@ pm_accept_to_pamtuples(int    const pipeToSuckFd,
 
     struct pamtuples * const outputTuplesP = accepterParm;
 
-    struct pam inpam;
+    struct pam * const inpamP = outputTuplesP->pamP;
 
     *outputTuplesP->tuplesP =
-        pnm_readpam(fdopen(pipeToSuckFd, "r"), &inpam, sizeof(inpam));
+        pnm_readpam(fdopen(pipeToSuckFd, "r"), inpamP,
+                    PAM_STRUCT_SIZE(tuple_type));
 
-    pm_close(inpam.file);
+    pm_close(inpamP->file);
 }
